function objDwPrefService() {
	///////////// Private variables /////////////
	var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
	var pref = prefService.getBranch(null);
	var dwPref = prefService.getBranch("downloadwith.");

	///////////// Private Methods /////////////
	function getApplicationList() {
		try { 
			return (dwPref.getCharPref("applicationList")); 
		} catch (e) { return ""; }
	}
 
	function setApplicationList(value) {
		dwPref.setCharPref("applicationList",value); 
	}

	///////////// Public Methods /////////////
	this.setFileTypesList = function (identification,value) {
		if (this.isApplication(identification)) {
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			try {
				dwPrefApplication.setCharPref("filetypes",value); 
			} catch(e) {}
		}
	}
 
	this.getFileTypesList = function (identification) {
		if (this.isApplication(identification)) {
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			try {
				return (dwPrefApplication.getCharPref("filetypes")); 
			} catch(e) { return ""; }
		}
		return "";
	}
 
	this.getIdentificationArray = function () {
		var aux = dwPref.getCharPref("applicationList");
		// Remove first ";" 
		aux = aux.replace(/^;/,"");
		// Remove last ";"
		aux = aux.replace(/;$/,"");
		if (aux != "") { return aux.split(";;"); }
		else { return []; }
	}

	this.isApplication = function (identification) {
	    var re = new RegExp(";" + identification + ";");
	    return(re.test(getApplicationList()));
	}

	this.getApplicationLabel = function (identification) {
		if (this.isApplication(identification)) {
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			try {
				return(dwPrefApplication.getCharPref("label"));
			} catch(e) {}
		}
		return "";
	}

	this.getApplicationCommand = function (identification) {
		if (this.isApplication(identification)) {
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			try {
				return(dwPrefApplication.getCharPref("command"));
			} catch(e) {}
		}
		return "";
	}

	this.getApplicationArguments = function (identification) {
		if (this.isApplication(identification)) {
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			try {return(dwPrefApplication.getCharPref("arguments"));}
			catch(e) {}
		}
		return "";
	}

	this.setApplication = function (identification,label,command,arguments) {
		if (this.isApplication(identification)) {
			var dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			dwPrefApplication.setCharPref("label", label);
			dwPrefApplication.setCharPref("command", command);
			dwPrefApplication.setCharPref("arguments", arguments);
			return true;
		}
		return false;
	}

	this.addApplication = function (identification,label,command,arguments) {
		if (!this.isApplication(identification)) { 
			setApplicationList(getApplicationList()+ ";" + identification + ";");
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			dwPrefApplication.setCharPref("label", label);
			dwPrefApplication.setCharPref("command", command);
			dwPrefApplication.setCharPref("arguments", arguments);
			return true;
		}
		return false;
	}

	this.delApplication = function (identification) {
		if (this.isApplication(identification)) { 
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			dwPrefApplication.deleteBranch("");
			var re = new RegExp(";" + identification + ";");
			setApplicationList(getApplicationList().replace(re,""));
			if (this.isAutoDownloadStatusAll() && identification == this.getAutoDownloadAllApplication()) 
			{
				this.setAutoDownloadAllApplication("");
			} 
			return true;
		}
		return false;
	}

	this.getIdentification = function (fileType) {
		var identificationArray = this.getIdentificationArray();
		var re = new RegExp(";" + fileType + ";");
		for (var i = 0; i < identificationArray.length; i++) {
			var identification = identificationArray[i];
			var fileTypeList = this.getFileTypesList(identification);
			if (re.test(fileTypeList)) return identification;
		}
		return "";
	}

	this.addFileType = function (identification,fileType) {
		fileType = fileType.toLowerCase();
		if (this.isApplication(identification)) { 
			this.delFileType(fileType); 
			var fileTypeList = this.getFileTypesList(identification);
			this.setFileTypesList(identification,fileTypeList + ";" + fileType + ";");
			return true;
		}
		return false;
	}

	this.delFileType = function (fileType) {
		var identificationArray = this.getIdentificationArray();
		var re = new RegExp(";" + fileType + ";");
		for (var i = 0; i < identificationArray.length; i++) {
			var identification = identificationArray[i];
			var fileTypeList = this.getFileTypesList(identification);
			if (re.test(fileTypeList)) {
				this.setFileTypesList(identification,fileTypeList.replace(re,""));
			}
		}
		return false;
	}

	this.getFileTypeArray = function (identification) {
		if (identification == null) {
			var aux = new Array ();
			var identificationArray = this.getIdentificationArray();
			for (var i = 0; i < identificationArray.length; i++) {
				var fileTypeArray = this.getFileTypeArray(identificationArray[i]);
				aux = aux.concat(fileTypeArray);
			}
			return aux.sort();
		} else if (this.isApplication(identification)) { 
			var aux = "";
			dwPrefApplication = prefService.getBranch("downloadwith." + identification + ".");
			try {
			  aux = dwPrefApplication.getCharPref("filetypes");
			} catch (e) {}
			// Remove first ";" 
			aux = aux.replace(/^;/,"");
			// Remove last ";"
			aux = aux.replace(/;$/,"");
			if (aux != "") { return aux.split(";;"); }
			else { return []; }
		}
		return [];
	}

	this.getAutoDownloadStatus = function () {
		return dwPref.getCharPref("autodownloadstatus");
	}

	this.setAutoDownloadStatusDisable = function () {
		dwPref.setCharPref("autodownloadstatus","disable");
	}

	this.setAutoDownloadStatusAll = function () {
		dwPref.setCharPref("autodownloadstatus","all");
		if (this.getAutoDownloadAllApplication() == "")
		{
			alert("No application configured, automatic download disabled");
			this.setAutoDownloadStatusDisable();
			return false;
		} 
		return true;
	}

	this.setAutoDownloadStatusSpecific = function () {
		dwPref.setCharPref("autodownloadstatus","specific");
	}

	this.isAutoDownloadStatusDisable = function () {
		if (dwPref.getCharPref("autodownloadstatus") == "disable") { return true; }
		else return false;
	}

	this.isAutoDownloadStatusAll = function () {
		if (dwPref.getCharPref("autodownloadstatus") == "all") { return true; }
		else return false;
	}

	this.isAutoDownloadStatusSpecific = function () {
		if (dwPref.getCharPref("autodownloadstatus") == "specific") { return true; }
		else return false;
	}

	this.getAutoDownloadAllApplication = function () {
		identification = dwPref.getCharPref("autodownloadapplication");
		if (this.isApplication(identification)) { return identification; }
		else { return ""; }
	}

	this.setAutoDownloadAllApplication = function (identification) {
		dwPref.setCharPref("autodownloadapplication",identification);
		if (identification == "" && this.isAutoDownloadStatusAll())
		{
			alert("No application configured, automatic download disabled");
			this.setAutoDownloadStatusDisable();
			return false;
		} 
		return true;
	}

	this.getContextMenuShowUrl = function () {
		return dwPref.getBoolPref("contextmenushowurl");
	}

	this.setContextMenuShowUrl = function (value) {
		dwPref.setBoolPref("contextmenushowurl",value);
	}
}